# -*- coding: utf-8 -*-
"""
DictionaryView.py - dictionary text view.

@Copyright (C) 2005 Nir Soffer <nirs@freeshell.org>
@license: GNU GPL, see COPYING for details.
"""

from Foundation import *
from AppKit import *
from PyObjCTools import NibClassBuilder

import dictionary

# Stylesheet - probably there is a better way to define this

fontSize = 14 # should be user preference
normalFont = NSFont.systemFontOfSize_(fontSize)
smallFont = NSFont.systemFontOfSize_(fontSize * 0.85)
    
termAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.blackColor(), NSForegroundColorAttributeName, None)

meaningAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    smallFont, NSFontAttributeName,
    NSColor.blackColor(), NSForegroundColorAttributeName, None)

errorAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.redColor(), NSForegroundColorAttributeName, None)

statusAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.grayColor(), NSForegroundColorAttributeName, None)


class DictionaryView(NibClassBuilder.AutoBaseClass):
    """ Dictionary text view 
    
    I'm using dictionary to define words and display results. Double 
    clicking on words will define them.
    """
    word = None
    hasDefinition = False

    def mouseDown_(self, event):
        """ Extend mouseDown to define words after double click """
        super(DictionaryView, self).mouseDown_(event)
        if event.clickCount() == 2 and self.selection():
            # A word was selected by double click
            NSApp().sendAction_to_from_('defineSelection:', None, self)
            
    def selection(self):
        return self.string().substringWithRange_(self.selectedRange()).strip()
                
    def define(self, word):
        self.word = word
        self.hasDefinition = False
        textStorage = self.textStorage()        
        textStorage.beginEditing()
        textStorage.mutableString().setString_(u'')
        try:
            definitions = dictionary.define(word.encode('utf-8'))
        except dictionary.Error, err:
            # TODO: make better error message, maybe add link to the site?
            self.error(NSLocalizedString(
                u'Translation failed. The server might be down.', None))
        else:
            if not definitions:
                # TODO: add link to the editor?                
                self.status(NSLocalizedString(u'Undefined word.', None))
            else:
                self.hasDefinition = True
                self.format(definitions)
        textStorage.endEditing()

    # ---------------------------------------------------------------------
    # Formatting results

    def format(self, definitions):
        for d in definitions:
            self.printString(u'%(he)s   %(en)s\n' % d, termAttributes)
            if d['meaning']:
                self.printString(u'%(meaning)s\n' % d, meaningAttributes)
            self.printString(u'\n')
        # Use always right to left, regardless of the user interface 
        # language, becuase the meaning text is only in Hebrew.
        self.setDirection(NSWritingDirectionRightToLeft)
        
    def error(self, message):        
        self.printString(message, errorAttributes)
        # TODO: use current user interface language direction
        self.setDirection(NSWritingDirectionLeftToRight)
        
    def status(self, message):
        self.printString(message, statusAttributes)
        # TODO: use current user interface language direction
        self.setDirection(NSWritingDirectionLeftToRight)

    def printString(self, string, attributes=None):
        textStorage = self.textStorage()
        range = NSMakeRange(textStorage.length(), 0)
        textStorage.replaceCharactersInRange_withString_(range, string)
        range.length = len(string)
        textStorage.setAttributes_range_(attributes, range)

    def setDirection(self, direction, range=None):
        textStorage = self.textStorage()
        range = range or NSMakeRange(0, textStorage.length())
        textStorage.addAttribute_value_range_(
            NSParagraphStyleAttributeName,
            self.paragraphStyleForDirection(direction),
            range)

    def paragraphStyleForDirection(self, direction):
        style = NSParagraphStyle.defaultParagraphStyle().mutableCopy()
        style.setBaseWritingDirection_(direction)
        if direction == NSWritingDirectionRightToLeft:
            style.setAlignment_(NSRightTextAlignment)
        else:
            style.setAlignment_(NSLeftTextAlignment)
        return style
        